@@ -21,6 +21,7 @@ KW_RESOURCE = 'resource'
 
 KW_CONTROLLER_RESOURCE = 'Controller'
 KW_SERVICE_RESOURCE = 'Service'
+KW_CLIENT_RESOURCE = 'Client'
 KW_REPOSITORY_RESOURCE = 'Repository'
 KW_VALIDATOR_RESOURCE = 'Validator'
 KW_MAPPER_RESOURCE = 'Mapper'
@@ -29,6 +30,7 @@ KW_CONVERTER_RESOURCE = 'Converter'
 KW_RESOURCE_LIST = [
     KW_CONTROLLER_RESOURCE,
     KW_SERVICE_RESOURCE,
+    KW_CLIENT_RESOURCE,
     KW_REPOSITORY_RESOURCE,
     KW_VALIDATOR_RESOURCE,
     KW_MAPPER_RESOURCE,
@@ -206,23 +208,23 @@ def bindResource(apiInstance,resourceInstance) :
     validateFlaskApi(apiInstance)
     setResource(getattr(apiInstance.resource, getResourceType(resourceInstance).lower()), resourceInstance)
 
-@Function
+@FunctionThrough
 def getGlobalException(exception, resourceInstance, resourceInstanceMethod):
     apiInstance = getNullableApi()
     return GlobalException.handleLogErrorException(exception, resourceInstance, resourceInstanceMethod, apiInstance)
 
-@Function
+@FunctionThrough
 def raiseGlobalException(exception, resourceInstance, resourceInstanceMethod) :
     raise getGlobalException(exception, resourceInstance, resourceInstanceMethod)
 
-@Function
+@FunctionThrough
 def getCompleteResponseByException(exception, resourceInstance, resourceInstanceMethod) :
     exception = getGlobalException(exception, resourceInstance, resourceInstanceMethod)
     completeResponse = [{'message':exception.message, 'timestamp':str(exception.timeStamp)},exception.status]
     log.error(resourceInstance.__class__, f'Error processing {resourceInstance.__class__.__name__}.{resourceInstanceMethod.__name__} request', exception)
     return completeResponse
 
-@Function
+@FunctionThrough
 def initialize(apiInstance, defaultUrl=None, openInBrowser=False) :
     defaultUrl = defaultUrl
     openInBrowser = openInBrowser
@@ -268,7 +270,7 @@ def Controller(
         return InnerClass
     return Wrapper
 
-@Function
+@FunctionThrough
 def getRequestBodyAsJson(contentType) :
     try :
         if OpenApiManager.DEFAULT_CONTENT_TYPE == contentType :
@@ -279,14 +281,14 @@ def getRequestBodyAsJson(contentType) :
         raise GlobalException.GlobalException(message='Not possible to parse the request', logMessage=str(exception), status=HttpStatus.BAD_REQUEST)
     return requestBodyAsJson
 
-@Function
+@FunctionThrough
 @Security.jwtRequired
 def securedMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, roleRequired, requestClass, logBodyRequest) :
     if not Security.getRole() in roleRequired :
         raise GlobalException.GlobalException(message='Role not allowed', logMessage=f'''Role {Security.getRole()} trying to access denied resourse''', status=HttpStatus.FORBIDEN)
     return notSecuredMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logBodyRequest)
 
-@Function
+@FunctionThrough
 def notSecuredMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logBodyRequest) :
     if resourceInstanceMethod.__name__ in OpenApiManager.ABLE_TO_RECIEVE_BODY_LIST and requestClass :
         requestBodyAsJson = getRequestBodyAsJson(contentType)
@@ -392,6 +394,7 @@ def Service() :
                 OuterClass.__init__(self,*args,**kwargs)
                 self.globals = apiInstance.globals
                 self.service = apiInstance.resource.service
+                self.client = apiInstance.resource.client
                 self.repository = apiInstance.resource.repository
                 self.validator = apiInstance.resource.validator
                 self.mapper = apiInstance.resource.mapper
@@ -418,6 +421,38 @@ def ServiceMethod(requestClass=None):
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
+@Function
+def Client() :
+    def Wrapper(OuterClass, *args, **kwargs):
+        apiInstance = getApi()
+        noException = None
+        log.debug(Client,f'''wrapping {OuterClass.__name__}''')
+        class InnerClass(OuterClass):
+            def __init__(self,*args,**kwargs):
+                log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
+                OuterClass.__init__(self,*args,**kwargs)
+                self.globals = apiInstance.globals
+        overrideSignatures(InnerClass, OuterClass)
+        return InnerClass
+    return Wrapper
+
+@Function
+def ClientMethod(requestClass=None):
+    def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
+        noException = None
+        log.debug(ClientMethod,f'''innerMethodWrapper wraped {resourceInstanceMethod.__name__}''')
+        def innerResourceInstanceMethod(*args,**kwargs) :
+            resourceInstance = args[0]
+            try :
+                validateArgs(args,requestClass,innerResourceInstanceMethod)
+                methodReturn = resourceInstanceMethod(*args,**kwargs)
+            except Exception as exception :
+                raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
+            return methodReturn
+        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        return innerResourceInstanceMethod
+    return innerMethodWrapper
+
 @Function
 def Repository(model = None) :
     repositoryModel = model
